package com.zanxiang.manage.service.Impl;

import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.UserDTO;
import com.zanxiang.manage.service.UserService;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.mapper.UserMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户玩家
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    /**
     * 根据玩家账号或者昵称模糊查询玩家列表
     *
     * @param userName : 玩家账号
     * @param nickname : 玩家昵称
     * @return : 返回玩家信息
     */
    @Override
    public List<UserDTO> getLikeUserNameOrNickName(String userName, String nickname) {
        List<User> userList = super.list(new LambdaQueryWrapper<User>()
                .like(Strings.isNotBlank(userName), User::getUsername, userName)
                .like(Strings.isNotBlank(nickname), User::getNickname, nickname)
                .select(User::getId, User::getUsername, User::getNickname));
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyList();
        }
        return BeanUtils.copyList(userList, UserDTO.class);
    }

    /**
     * 根据用户id查询信息
     *
     * @param userId : 返回用户信息
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getById(Long userId) {
        return BeanUtils.copy(super.getById(userId), UserDTO.class);
    }

}
